package com.gree.mall.contest.bean.zfire;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.gree.mall.contest.plus.entity.AdminField;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


@Data
public class ZfireParamBean {

    @Schema(description = "页号，不传默认1")
    private Integer pageNum = 1;

    @Schema(description = "页大小，不传默认10，查全部传 -1")
    private Integer pageSize = 10;

    @Schema(description = "排序字段例如：table.create_time desc")
    private String orderBy = "";

    @Schema(description = "查询条件")
    private List<QueryParamBean> params;

    @Schema(description = "导出的字段,导出必传")
    private List<AdminField> exportFields;

    @Schema(description = "预留字段(前端忽略该字段)")
    private String query;

    @Schema(description = "查询项")
    private String selected;


    @Schema(description = "导出用字段")
    private Integer exportCount = 0;

    @JsonIgnore
    @Schema(description = "返回类的类型")
    private Class clazzType;

    @JsonIgnore
    @Schema(description = "商户字段")
    private String companyWechatId;

    @JsonIgnore
    private List<String> adminWebsitIds;

    public Page page(){
        return new Page(pageNum,pageSize);
    }

    /**
     * 查询指定条件内容
     */
    public Object getParamValue(String param){
        if(CollectionUtils.isEmpty(params))
            return null;
        List<QueryParamBean> collect = params.stream().filter(v -> v.getParam().equals(param)).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(collect)){
            return null;
        }
        return collect.get(0).getValue();
    }

    /**
     * 指定查询条件内容
     */
    public Boolean setParamValue(String param,String compare,Object value){
        if(CollectionUtils.isEmpty(params)) {
            params = new ArrayList<>();
        }
        //1. if exists remove
        this.removeParam(param);
        //2. add
        QueryParamBean queryParamBean = new QueryParamBean();
        queryParamBean.setParam(param);
        queryParamBean.setCompare(compare);
        queryParamBean.setValue(value);
        params.add(queryParamBean);
        return true;
    }

    /**
     * 删除指定查询条件
     */
    public Boolean removeParam(String param){
        if(CollectionUtils.isEmpty(params)) {
            return true;
        }
        params.removeIf(v -> v.getParam().equals(param));
        return true;
    }
}
