package com.gree.mall.contest.config;

import org.springdoc.core.GroupedOpenApi;
import org.springdoc.core.customizers.OpenApiCustomiser;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.awt.*;
import java.io.IOException;
import java.net.URI;

@Configuration
public class SwaggerUIConfig {
    @Value("${server.port}")
    private String port;
    @Value("${spring.profiles.active}")
    private String active;
    @Value("${server.servlet.context-path}")
    private String path;
    @Bean
    public OpenApiCustomiser globalHeaderCustomizer() {
        return openApi -> openApi.getPaths()
                .values()
                .forEach(pathItem -> pathItem.readOperations()
                        .forEach(operation -> operation.addParametersItem(
                                new io.swagger.v3.oas.models.parameters.Parameter()
                                        .in("header")
                                        .name("x-token")
                                        .required(true)
                                        .example("1111")
                                        .description("认证令牌")
                        )));
    }

    @Bean
    public GroupedOpenApi pcApi() {
        return GroupedOpenApi.builder()
                .group("管理端接口")
                .displayName("管理端接口")
                .pathsToMatch("/pc/**")
                .build();
    }

    @Bean
    public GroupedOpenApi miniApi() {
        return GroupedOpenApi.builder()
                .group("移动端接口")
                .displayName("移动端接口")
                .pathsToMatch("/mini/**")
                .build();
    }

    @Bean
    public CommandLineRunner openBrowser() {
        return args -> {
            try {
                String swaggerPath = "/doc.html"; // Springdoc路径
                String url = "http://localhost:" + port + path + swaggerPath;
                System.out.println("尝试打开Swagger UI: " + url);

                if (Desktop.isDesktopSupported()) {
                    Desktop desktop = Desktop.getDesktop();
                    if (desktop.isSupported(Desktop.Action.BROWSE)) {
                        desktop.browse(new URI(url));
                        return;
                    }
                }

                // Desktop不可用时使用命令行
                System.out.println("使用命令行方式打开...");
                openUrlWithCommand(url);

            } catch (Exception e) {
                System.err.println("无法打开浏览器: " + e.getMessage());
                e.printStackTrace();
            }
        };
    }

    private void openUrlWithCommand(String url) throws IOException {
        String os = System.getProperty("os.name").toLowerCase();
        Runtime runtime = Runtime.getRuntime();

        if (os.contains("win")) {
            runtime.exec("rundll32 url.dll,FileProtocolHandler " + url);
        } else if (os.contains("mac")) {
            runtime.exec("open " + url);
        } else if (os.contains("nix") || os.contains("nux")) {
            runtime.exec("xdg-open " + url);
        } else {
            System.out.println("请手动访问: " + url);
        }
    }
}
