package com.gree.mall.contest.controller.pc.zfire;

import cn.hutool.core.bean.BeanUtil;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.zfire.QueryParamBean;
import com.gree.mall.contest.bean.zfire.ZfireCustomParam;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.AdminField;
import com.gree.mall.contest.plus.entity.AdminFieldCustomParam;
import com.gree.mall.contest.plus.entity.AdminFieldCustomParamItem;
import com.gree.mall.contest.plus.service.AdminFieldCustomParamItemService;
import com.gree.mall.contest.plus.service.AdminFieldCustomParamService;
import com.gree.mall.contest.plus.service.AdminFieldService;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Slf4j
@RestController
@RequestMapping("/pc/zfire")
@Tag(name = "通用列表API", description = "通用列表API")
@RequiredArgsConstructor
public class ZfireController {

    private final CommonLogic commonLogic;
    private final AdminFieldService adminFieldService;
    private final AdminFieldCustomParamService adminFieldCustomParamService;
    private final AdminFieldCustomParamItemService adminFieldCustomParamItemService;

    @PostMapping("save")
    @Schema(description = "保存配置的列表字段")
    public ResponseHelper<List<AdminField>> save(@RequestBody List<AdminField> adminFieldList) {
        if (CollectionUtils.isEmpty(adminFieldList)) {
            return ResponseHelper.success();
        }
        String adminUserId = adminFieldList.get(0).getAdminUserId();
        String moduleId = adminFieldList.get(0).getModuleId();
        adminFieldService.lambdaUpdate().eq(AdminField::getAdminUserId, adminUserId).eq(AdminField::getModuleId, moduleId).remove();
        adminFieldService.saveBatch(adminFieldList);
        return ResponseHelper.success(adminFieldList);
    }

    @PostMapping("delete")
    @Schema(description = "删除配置的列表字段")
    public ResponseHelper save(@RequestParam String adminUserId, @RequestParam String moduleId) {
        adminFieldService.lambdaUpdate().eq(AdminField::getAdminUserId, adminUserId).eq(AdminField::getModuleId, moduleId).remove();
        return ResponseHelper.success();
    }

    @PostMapping("param/save")
    @Schema(description = "自定义的查询条件-保存")
    public ResponseHelper saveParam(@Valid @RequestBody ZfireCustomParam zfireCustomParam) {
        if (CollectionUtils.isEmpty(zfireCustomParam.getItems())) {
            throw new RemoteServiceException("缺少必要的参数items");
        }
        if (zfireCustomParam.getName().length() > 8) {
            throw new RemoteServiceException("自定义查询方案名称长度不可超过8");
        }
        AdminUserCom adminUser = commonLogic.getAdminUser();
        if (StringUtils.isBlank(zfireCustomParam.getId())) {
            Long count = adminFieldCustomParamService.lambdaQuery()
                    .eq(AdminFieldCustomParam::getAdminUserId, adminUser.getAdminUserId())
                    .eq(AdminFieldCustomParam::getModuleId, zfireCustomParam.getModuleId())
                    .count();
            if (count >= 10) {
                throw new RemoteServiceException("最多支持10个自定义查询方案");
            }
        }
        AdminFieldCustomParam adminFieldCustomParam = new AdminFieldCustomParam();
        adminFieldCustomParam.setId(zfireCustomParam.getId());
        adminFieldCustomParam.setName(zfireCustomParam.getName());
        adminFieldCustomParam.setModuleId(zfireCustomParam.getModuleId());
        adminFieldCustomParam.setAdminUserId(adminUser.getAdminUserId());
        adminFieldCustomParamService.saveOrUpdate(adminFieldCustomParam);
        adminFieldCustomParamItemService.lambdaUpdate().eq(AdminFieldCustomParamItem::getAdminFieldCustomId, adminFieldCustomParam.getId()).remove();
        List<AdminFieldCustomParamItem> items = BeanUtil.copyToList(zfireCustomParam.getItems(), AdminFieldCustomParamItem.class);
        items.forEach(v -> v.setAdminFieldCustomId(adminFieldCustomParam.getId()));
        adminFieldCustomParamItemService.saveBatch(items);
        return ResponseHelper.success();
    }

    @PostMapping("param/delete")
    @Schema(description = "自定义的查询条件-删除")
    public ResponseHelper deleteParam(@RequestParam String id) {
        adminFieldCustomParamItemService.lambdaUpdate().eq(AdminFieldCustomParamItem::getAdminFieldCustomId, id).remove();
        adminFieldCustomParamService.removeById(id);
        return ResponseHelper.success();
    }

    @PostMapping("param/list")
    @Schema(description = "自定义的查询条件-查询")
    public ResponseHelper<List<ZfireCustomParam>> listParam(@RequestParam String moduleId) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        List<AdminFieldCustomParam> list = adminFieldCustomParamService.lambdaQuery()
                .eq(AdminFieldCustomParam::getAdminUserId, adminUser.getAdminUserId())
                .eq(AdminFieldCustomParam::getModuleId, moduleId).list();
        List<ZfireCustomParam> zfireCustomBeans = BeanUtil.copyToList(list, ZfireCustomParam.class);
        for (ZfireCustomParam bean : zfireCustomBeans) {
            List<AdminFieldCustomParamItem> items = adminFieldCustomParamItemService.lambdaQuery().eq(AdminFieldCustomParamItem::getAdminFieldCustomId, bean.getId()).list();
            List<QueryParamBean> newItems = BeanUtil.copyToList(items, QueryParamBean.class);
            bean.setItems(newItems);
        }
        return ResponseHelper.success(zfireCustomBeans);
    }

}
